package cn.ibizlab.core.analysis.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.core.analysis.domain.DABuild;
import cn.ibizlab.core.analysis.filter.DABuildSearchContext;
import cn.ibizlab.core.analysis.service.IDABuildService;

import cn.ibizlab.util.helper.CachedBeanCopier;
import cn.ibizlab.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.core.analysis.mapper.DABuildMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[分析] 服务对象接口实现
 */
@Slf4j
@Service("DABuildServiceImpl")
public class DABuildServiceImpl extends ServiceImpl<DABuildMapper, DABuild> implements IDABuildService {

    @Autowired
    @Lazy
    protected cn.ibizlab.core.analysis.service.IDAMetricService dametricService;
    @Autowired
    @Lazy
    protected cn.ibizlab.core.lite.service.IMetaModelService metamodelService;
    @Autowired
    @Lazy
    IDABuildService proxyService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(DABuild et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        dametricService.saveByBuildId(et.getBuildId(), et.getMetrics());
        CachedBeanCopier.copy(get(et.getBuildId()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<DABuild> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(DABuild et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("buildid", et.getBuildId()))) {
            return false;
        }
        dametricService.saveByBuildId(et.getBuildId(), et.getMetrics());
        CachedBeanCopier.copy(get(et.getBuildId()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<DABuild> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        dametricService.removeByBuildId(key) ;
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public DABuild get(String key) {
        DABuild et = getById(key);
        if(et == null){
            et = new DABuild();
            et.setBuildId(key);
        }
        else {
            et.setMetrics(dametricService.selectByBuildId(key));
        }
        return et;
    }

    @Override
    public DABuild getDraft(DABuild et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(DABuild et) {
        return (!ObjectUtils.isEmpty(et.getBuildId())) && (!Objects.isNull(this.getById(et.getBuildId())));
    }
    @Override
    @Transactional
    public DABuild exportZip(DABuild et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean exportZipBatch(List<DABuild> etList) {
        for(DABuild et : etList) {
            exportZip(et);
        }
        return true;
    }

    @Override
    @Transactional
    public DABuild run(DABuild et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean runBatch(List<DABuild> etList) {
        for(DABuild et : etList) {
            run(et);
        }
        return true;
    }

    @Override
    @Transactional
    public boolean save(DABuild et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(DABuild et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? proxyService.update(et) : proxyService.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<DABuild> list) {
        list.forEach(item->fillParentData(item));
        List<DABuild> create = new ArrayList<>();
        List<DABuild> update = new ArrayList<>();
        for (DABuild et : list) {
            if (ObjectUtils.isEmpty(et.getBuildId()) || ObjectUtils.isEmpty(getById(et.getBuildId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<DABuild> list) {
        list.forEach(item->fillParentData(item));
        List<DABuild> create = new ArrayList<>();
        List<DABuild> update = new ArrayList<>();
        for (DABuild et : list) {
            if (ObjectUtils.isEmpty(et.getBuildId()) || ObjectUtils.isEmpty(getById(et.getBuildId()))) {
                create.add(et);
            } else {
                update.add(et);
            }
        }
        if (create.size() > 0) {
            proxyService.createBatch(create);
        }
        if (update.size() > 0) {
            proxyService.updateBatch(update);
        }
    }


	@Override
    public List<DABuild> selectByModelId(String id) {
        return baseMapper.selectByModelId(id);
    }
    @Override
    public void removeByModelId(String id) {
        this.remove(new QueryWrapper<DABuild>().eq("modelid",id));
    }

	@Override
    public void saveByModelId(String id,List<DABuild> list) {
        if(list==null)
            return;
        Set<String> delIds=new HashSet<String>();
        List<DABuild> _update=new ArrayList<DABuild>();
        List<DABuild> _create=new ArrayList<DABuild>();
        for(DABuild before:selectByModelId(id)){
            delIds.add(before.getBuildId());
        }
        for(DABuild sub:list) {
            sub.setModelId(id);
            if(ObjectUtils.isEmpty(sub.getBuildId()))
                sub.setBuildId((String)sub.getDefaultKey(true));
            if(delIds.contains(sub.getBuildId())) {
                delIds.remove(sub.getBuildId());
                _update.add(sub);
            }
            else
                _create.add(sub);
        }
        if(_update.size()>0)
            proxyService.updateBatch(_update);
        if(_create.size()>0)
            proxyService.createBatch(_create);
        if(delIds.size()>0)
            proxyService.removeBatch(delIds);
	}


    /**
     * 查询集合 数据集
     */
    @Override
    public Page<DABuild> searchDefault(DABuildSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<DABuild> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<DABuild>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(DABuild et){
        //实体关系[DER1N_DA_BUILD_META_MODEL_MODELID]
        if(!ObjectUtils.isEmpty(et.getModelId())){
            cn.ibizlab.core.lite.domain.MetaModel model=et.getModel();
            if(ObjectUtils.isEmpty(model)){
                cn.ibizlab.core.lite.domain.MetaModel majorEntity=metamodelService.get(et.getModelId());
                et.setModel(majorEntity);
                model=majorEntity;
            }
            et.setModelName(model.getName());
            et.setSystemId(model.getSystemId());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }





}



